from clitellum.channels.configuration import ChannelConfiguration
from clitellum.handlers import HandlerManager
from clitellum.services import ServiceFactory


def microservice(bc, micro_type, micro_id, package, bc_publisher=None):
    def decorator(f):
        MicroServiceBuilder.get_instance().set_bc(bc)
        MicroServiceBuilder.get_instance().set_micro_type(micro_type)
        MicroServiceBuilder.get_instance().set_micro_id(micro_id)
        MicroServiceBuilder.get_instance().set_package(package)
        MicroServiceBuilder.get_instance().set_bc_publisher(bc_publisher)
        return f
    return decorator


class MicroServiceBuilder:
    _instance = None

    @classmethod
    def get_instance(cls):
        if cls._instance is None:
            cls._instance = MicroServiceBuilder()
        return cls._instance

    def __init__(self):
        self.__bc = None
        self.__micro_type = None
        self.__micro_id = None
        self.__package = None
        self.__bc_publisher = None
        self.__amqp_uri = "amqp://guest:guest@localhost:5672/"
        self.__max_threads = 4

    def set_bc(self, bc):
        self.__bc = bc

    def set_micro_type(self, micro_type):
        self.__micro_type = micro_type

    def set_micro_id(self, micro_id):
        self.__micro_id = micro_id

    def set_package(self, package):
        self.__package = package

    def set_bc_publisher(self, bc_publisher):
        self.__bc_publisher = bc_publisher

    def set_amqp_uri(self, amqp_uri):
        self.__amqp_uri = amqp_uri

    def set_max_thread(self, max_threads):
        self.__max_threads = max_threads

    def build(self):
        # Load handlers packages
        __import__(self.__package)

        config_receiver = ChannelConfiguration()
        config_receiver.uri = self.__amqp_uri
        config_receiver.exchange.name = self.__bc
        config_receiver.queue.name = '%s.%s.%s' % (self.__bc, self.__micro_type, self.__micro_id)

        for routing_key in HandlerManager.get_instance().get_routing_keys():
            config_receiver.queue.add_routing_key(routing_key)

        config_receiver.max_threads = 4

        config_sender = ChannelConfiguration()
        config_sender.uri = self.__amqp_uri
        config_sender.exchange.name = self.__bc_publisher if self.__bc_publisher is not None else self.__bc

        service = ServiceFactory.create_service(service_id=self.__micro_id,
                                                service_type=self.__micro_type,
                                                config_sender=config_sender,
                                                config_receiver=config_receiver,
                                                handler_manager=HandlerManager.get_instance())

        return service




import socket
import threading
from concurrent.futures.thread import ThreadPoolExecutor

from clitellum import logger_manager
from clitellum.channels.amqp import AmqpConnection, AmqpMessageProperties, AmqpMessage
from clitellum.channels.configuration import ChannelConfiguration


class Channel:

    @classmethod
    def create(cls, config: ChannelConfiguration):
        """
        Factory Method para crear channels
        :param config: Configuracion del chanel
        :return:
        """
        conn = AmqpConnection(config)
        channel = Channel(conn, config)
        return channel

    def __init__(self, connection: AmqpConnection, config: ChannelConfiguration):
        self._config = config
        self._connection = connection
        self._connection.on_message_received = self._message_received
        self.on_message_received = None
        self._thread_pool = ThreadPoolExecutor(max_workers=config.max_threads)
        self._thread_consumer = None
        self._is_consumming = False

    def _message_received(self, message: AmqpMessage):
        if self.on_message_received is not None:
            self._thread_pool.submit(self.on_message_received, message)
        #self._connection.ack(message)

    def connect(self):
        self._connection.connect()

    def close(self):
        if self._is_consumming:
            self.stop_consuming()
        self._connection.close()

    def publish(self, message: bytes, routing_key: str, properties: AmqpMessageProperties):
        self._connection.publish(message, routing_key, properties)

    def consume(self):
        self._is_consumming = True
        #self._worker_consumer()
        self._thread_consumer = threading.Thread(target=self._worker_consumer)
        self._thread_consumer.start()

    def _worker_consumer(self):
        self._connection.consume()
        while self._is_consumming:
            try:
                self._connection.drain_events(timeout=10)
            except socket.timeout:
                logger_manager.get_logger().debug("Check consuming status")
            except Exception as ex:
                logger_manager.get_logger().error("Error start consuming", ex)

        logger_manager.get_logger().debug("Fin del hilo de consumicion")
        self._connection.basic_cancel()

    def stop_consuming(self):
        self._is_consumming = False
        self._thread_consumer.join()
        self._thread_pool.shutdown()

    def ack(self, message: AmqpMessage):
        self._connection.ack(message)



import re
from enum import Enum


class AmqpElementConfiguration:
    """
    Clase base de configuracion de un elemento amqp
    como un exchange o una cola
    """
    def __init__(self):
        self.name = None
        self.create = True


class ExhangeType(Enum):
    DIRECT = "direct"
    TOPIC = "topic"


class ExchangeConfiguration(AmqpElementConfiguration):
    """
    Configuracion de un exchange
    """
    def __init__(self):
        AmqpElementConfiguration.__init__(self)
        self.type = ExhangeType.TOPIC

    def set_exchange_direct(self):
        """
        Establece el exchange de tipo direct
        """
        self.type = ExhangeType.DIRECT

    def set_exchange_topic(self):
        """
        Establece el exchange de tipo topic
        """
        self.type = ExhangeType.TOPIC


class QueueConfiguration(AmqpElementConfiguration):
    """
    Configuracion de una cola
    """
    def __init__(self):
        AmqpElementConfiguration.__init__(self)
        self._routingKeys = list()

    def add_routing_key(self, key):
        """
        Añade una clave de enrutamiento de la cola amqp
        :param key: Clave de enrutamiento
        """
        self._routingKeys.append(key)

    def get_routing_keys(self) -> list:
        """
        Devuelve la lista de claves de enrutamiento de la cola
        :return: Lista de claves
        :rtype: list
        """
        return self._routingKeys


class ConnectionParameters:
    """
    Contiene los parametros de conexion al servidor AMQP
    """
    @classmethod
    def create_from_uri(cls, uri: str):
        """
        Factory Method, crea extrae los parametros de la conexion a partir
        de una uri amqp://guest:guest@localhost:5672/
        :param uri: Url de conexion a AMQP
        :return: Propiedades de la conexion
        :rtype: clitellum.channels.configuration.ConnectionParameters
        """
        params = ConnectionParameters()
        m = re.search('^amqp:\/\/((\w*):(\w*))?@?(([\w\.]*)(:(\d*))?)\/([\w%]*)', uri)
        params.host = m.group(5)

        if m.group(7) is not None and m.group(7) is not '':
            params.port = int(m.group(7))

        if m.group(2) is not None and m.group(2) is not '':
            params.userid = m.group(2)

        if m.group(3) is not None and m.group(3) is not '':
            params.password = m.group(3)

        if m.group(8) is not None and m.group(8) is not '':
            params.virtual_host = m.group(8)

        return params

    def __init__(self):
        self.host = 'localhost'
        self.userid = 'guest'
        self.password = 'guest'
        self.login_method = 'AMQPLAIN'
        self.virtual_host = '/'
        self.port = 5672


class ChannelConfiguration:
    """
    Congiguracion de un canal de comunicacion amqp
    """
    def __init__(self):
        self.uri = None
        self.exchange = ExchangeConfiguration()
        self.queue = QueueConfiguration()
        self.max_threads = 4

    def get_connection_parameters(self) -> ConnectionParameters:
        """
        Devuelve los parametros de la conexion
        :return: Parametros de la conexion
        :rtype: clitellum.channels.configuration.ConnectionParameters
        """
        return ConnectionParameters.create_from_uri(self.uri)



import socket

import librabbitmq

from clitellum import logger_manager

from clitellum.channels.configuration import ChannelConfiguration
from clitellum.exceptions import ConnectionException
import threading


class AmqpMessageProperties:
    """
    Propiedades de los mensajes amqp
    """
    def __init__(self):
        self.content_type = 'application/json'
        self.content_encoding = 'utf-8'
        self.headers = dict()
        self.delivery_mode = 2
        self.priority = 0
        self.correlation_id = None
        self.reply_to = None
        self.expiration = None
        self.app_id = None
        self.message_id = None

    def to_dict(self):
        """
        Convierte la propiedades en un objeto del tipo pika BasicProperties
        para el envio del mensaje
        :return: pika.BasicProperties
        """
        properties = dict()
        if self.content_type is not None:
            properties['content_type'] = self.content_type

        if self.content_encoding is not None:
            properties['content_encoding'] = self.content_encoding

        if self.headers is not None and len(self.headers) > 0:
            properties['headers'] = self.headers

        properties['delivery_mode'] = self.delivery_mode
        properties['priority'] = self.priority

        if self.correlation_id is not None:
            properties['correlation_id'] = self.correlation_id

        if self.reply_to is not None:
            properties['reply_to'] = self.reply_to

        if self.expiration is not None:
            properties['expiration'] = self.expiration

        if self.app_id is not None:
            properties['app_id'] = self.app_id

        if self.message_id is not None:
            properties['message_id'] = self.message_id

        return properties

    @classmethod
    def from_basic_properties(cls, properties):
        """
        Crea una instancia de AmqpMessageProperties a partir de las BasicProperties
        recibidas
        :param properties: propiedades
        :return:
        """
        amp = AmqpMessageProperties()
        amp.content_encoding = properties['content_encoding'] if 'content_encoding' in properties else None
        amp.message_id = properties['message_id'] if 'message_id' in properties else None
        amp.reply_to = properties['reply_to'] if 'reply_to' in properties else None
        amp.app_id = properties['app_id'] if 'app_id' in properties else None
        amp.expiration = properties['expiration'] if 'expiration' in properties else None
        amp.correlation_id = properties['correlation_id'] if 'correlation_id' in properties else None
        amp.priority = properties['priority'] if 'priority' in properties else None
        amp.headers = properties['headers'] if 'headers' in properties else None
        amp.content_type = properties['content_type'] if 'content_type' in properties else None
        return amp

    @classmethod
    def create(cls):
        properties = AmqpMessageProperties()
        properties.delivery_mode = 2
        return properties


class AmqpMesageFrame:

    @classmethod
    def from_message_frame(cls, frame):
        """
        Crea una instance de AmqpMessageFrame a partir de las propiedades
        del mensaje que se ha recibido
        :param frame: Basic.Deliver
        :return: AmqpMessageFrame
        """
        amf = AmqpMesageFrame()
        amf.exchange = frame['exchange'] if 'exchange' in frame else None
        amf.consumer_tag = frame['consumer_tag'] if 'consumer_tag' in frame else None
        amf.delivery_tag = frame['delivery_tag'] if 'delivery_tag' in frame else None
        amf.routing_key = frame['routing_key'] if 'routing_key' in frame else None
        amf.redelivered = frame['redelivered'] if 'redelivered' in frame else None
        return amf

    def __init__(self):
        self.consumer_tag = None
        self.delivery_tag = None
        self.routing_key = None
        self.exchange = None
        self.redelivered = None


class AmqpMessage:

    @classmethod
    def create_from_message(cls, message):
        """
        Crea una instancia de AmqpMessage a partir del mensaje recibido
        :param message:
        :return:
        :rtype: clitellum.channels.amqp.AmqpMessage
        """
        amf = AmqpMesageFrame.from_message_frame(message.delivery_info)
        amp = AmqpMessageProperties.from_basic_properties(message.properties)
        return AmqpMessage(amf, amp, message.body.tobytes())

    def __init__(self, frame: AmqpMesageFrame, properties: AmqpMessageProperties, body: bytes):
        self.frame = frame
        self.properties = properties
        self.body = body


class AmqpConnection:
    """
    Conexion a amqp
    """
    def __init__(self, configuration: ChannelConfiguration):
        self._configuration = configuration
        """
        :type: channels.configuration.ChannelConfiguration
        """

        self._connection = None
        """
        :type: amqp.connection.Connection
        """

        self._isConnected = False
        self._channel = None
        """
        :type: librabbitmq.Channel
        """
        self._closing = False
        self._consumer_tag = None
        self._channel_send = None
        """
        :type: librabbitmq.Channel
        """
        self._mutex = threading.Lock()
        self._mutex_ack = threading.Lock()

        self.on_message_received = None

    def connect(self):
        """
        Realiza la conexion con el host
        """

        if self._isConnected:
            return

        self._isConnected = False

        while not self._closing:
            try:
                logger_manager.get_logger().info('Conectando a %s', self._configuration.uri)
                self._connect_point()
                self._isConnected = True
                return
            except ConnectionError as ex:
                logger_manager.get_logger().error("Error al conectar: %s", ex)
                logger_manager.get_logger().error("Intentado reconectar... Host: %s", self._configuration.uri)

    def _connect_point(self):
        try:
            if self._connection is not None and self._connection.is_open:
                self._connection.close()

            parameters = self._configuration.get_connection_parameters()

            logger_manager.get_logger().debug('Creando conexion')
            self._connection = librabbitmq.Connection(
                host=parameters.host,
                userid=parameters.userid,
                password=parameters.password,
                virtual_host=parameters.virtual_host
            )

            logger_manager.get_logger().debug('Creando channel')
            self._channel = self._connection.channel(1)
            self._channel_send = self._connection.channel(2)

            if self._configuration.exchange.create:
                logger_manager.get_logger().debug('Creando exchange')
                self._channel.exchange_declare(self._configuration.exchange.name,
                                               self._configuration.exchange.type.value,
                                               durable=True,
                                               auto_delete=False)

            if self._configuration.queue.create and self._configuration.queue.name is not None:
                logger_manager.get_logger().debug('Creando cola')
                self._channel.queue_declare(queue=self._configuration.queue.name,
                                            durable=True, auto_delete=False)

                for key in self._configuration.queue.get_routing_keys():
                    self._channel.queue_bind(queue=self._configuration.queue.name,
                                             exchange=self._configuration.exchange.name,
                                             routing_key=key)
            self._channel.basic_qos(0, 100, True)

        except Exception as ex:
            logger_manager.get_logger().error('Error Creando conexion', ex)
            raise ConnectionException(ex)

    def close(self):
        """
        Cierra la conexion del channel
        """
        logger_manager.get_logger().info('Cerrando conexion')
        self._closing = True
        try:
            if self._isConnected:
                self._channel.close()
                self._channel_send.close()
                self._connection.close()
                self._connection = None
        except Exception as ex:
            logger_manager.get_logger().error("Error al cerrar: %s", ex)

        self._isConnected = False

    def publish(self, message: bytes, routing_key: str, amqp_message_properties: AmqpMessageProperties):
        """
        Publica un mensaje en el amqp
        :param amqp_message_properties: Propiedades del mensaje
        :param message: Cadena con el mensaje
        :param routing_key: Clave de enrutamiento del mensaje
        """
        self._mutex.acquire()
        while not self._closing:
            try:
                properties = amqp_message_properties.to_dict()
                self._channel_send.basic_publish((message, properties),
                                                 exchange=self._configuration.exchange.name,
                                                 routing_key=routing_key)
                break
            except Exception as ex:
                logger_manager.get_logger().error("Error de conexion al enviar el mensaje %s" % ex)
                self.connect()

        self._mutex.release()

    def consume(self):
        self._consumer_tag = self._channel.basic_consume(queue=self._configuration.queue.name,
                                                         callback=self.__read_message)
        logger_manager.get_logger().info("Start Consuming %s", self._consumer_tag)

    def drain_events(self, timeout):
        self._connection.drain_events(timeout=timeout)

    def __read_message(self, message):
        logger_manager.get_logger().debug("Message Received %s", message.body.tobytes())
        amqp_message = AmqpMessage.create_from_message(message)
        self.on_message_received(amqp_message)

    def ack(self, message: AmqpMessage):
        """
        Realiza el ack del mensaje en el servidor amqp
        :param message: Mensaje del que se realiza el ack
        """

        try:
            self._mutex_ack.acquire()
            self._channel.basic_ack(delivery_tag=message.frame.delivery_tag, multiple=False)
        except Exception as ex:
            logger_manager.get_logger().error("Error al enviar el ack", ex)
        finally:
            self._mutex_ack.release()

    def basic_cancel(self):
        self._channel.basic_cancel(self._consumer_tag)



from setuptools import setup

setup(
    name='clitellum',
    version='5.0.3',
    packages=['clitellum', 'clitellum.channels'],
    package_dir={'clitellum': 'clitellum'},
    url='https://clitellum.hotaka.io',
    license='GPL',
    author='Sergio.Bermudez',
    author_email='sergio@hotaka.io',
    description='Clitellum Microservice Framework',
    extras_require={
    },
    install_requires=['librabbitmq']
)



from clitellum.services import Context, Publisher


def handler(key):
    def decorator(f):
        HandlerManager.get_instance().add_handler(key, f)
        return f
    return decorator


class HandlerManager:
    """
    Controla los handlers de los mensajes
    """
    _instance = None
    @classmethod
    def get_instance(cls):
        if cls._instance is None:
            cls._instance = HandlerManager()
        return cls._instance

    def __init__(self):
        self.__handlers = dict()

    def add_handler(self, key, class_type):
        self.__handlers[key] = class_type

    def get_handler(self, key):
        """
        Devuelve el putero a la clase del handler que controla el tipo de mensaje
        :param key: Clave de enrutamiento
        :return:
        """
        return self.__handlers[key]

    def get_routing_keys(self):
        """
        Devuelve la lista de routing keys de los handlers
        :return: routing key list
        """
        return self.__handlers.keys()


class CiltellumHandler:
    """
    Clase base para los handlers
    """
    def __init__(self):
        self.__message = None
        """
        :rtype: dict
        """
        self.__publisher = None
        """
        :rtype: clitellum.services.Publisher
        """
        self.__context = None
        """
        :rtype: clitellum.services.Context
        """

    def initialize(self, publisher: Publisher, context: Context, message: dict):
        """
        Inicializa el handler con los parametros del mensaje amqp
        :param publisher: Publicador
        :param context: Contexto del mensaje
        :param message: Mensaje Amqp
        :return:
        """
        self.__message = message
        self.__publisher = publisher
        self.__context = context

    def _get_message(self):
        """
        Devuelve el mensaje
        :return: Mensaje
        """
        return self.__message

    def _get_header(self, key: str) -> str:
        """
        Devuelve la cabecera asociada a la clave
        :param key: Clave de la cabecera
        :return: Valor de la cabecera
        :rtype: str
        """
        return self.__context.get_item(key)

    def _add_header(self, key: str, value: str):
        """
        Añade una cabecera o la actualiza
        :param key: Clave de la cabecera
        :param value: Valor de la cabecera
        """
        self.__context.add_header(key, value)

    def handle(self):
        pass

    def _publish(self, message, routing_key: str):
        """
        Publica un mensaje en el broker
        :param message: Mensaje a enviar en string
        :param routing_key: Clave de enrutamiento
        """
        self.__publisher.publish(message, routing_key, self.__context)




from clitellum.channels.amqp import AmqpMessage, AmqpMessageProperties
from clitellum.channels.channel import Channel
import json
from clitellum import logger_manager

from clitellum.channels.configuration import ChannelConfiguration


class Identification:
    """
    Clase de identificacion de los servicios
    """
    def __init__(self, _id: str, _type: str):
        """
        Constructor
        :param _id: Id del servicio
        :param _type: Tipo del servicio
        """
        self.type = _type
        self.id = _id


class Context:
    """
    Contiene la informacion del contexto de ejecución del mensaje
    """

    def __init__(self, properties: AmqpMessageProperties):
        """
        Crea una instancia del contexto
        :param properties: Propiedades del contexto
        """
        self.properties = properties

    def get_headers(self) -> dict:
        """
        Devuelve el diccionario de las cabeceras
        :return: Diccionario
        :rtype: dict
        """
        return self.properties.headers

    def get_item(self, key: str) -> str:
        """
        Devuelve el elemento en funcion de su clave, si la clave no existe devuelve None
        :param key: Clave del elemento
        :return: Cadena con el elemento o None
        :rtype: str
        """
        return self.properties.headers[key] if key in self.properties.headers else None

    def add_header(self, key: str, value: str):
        """
        Añade o actualiza una cabecera en el contexto
        :param key: Clave de la cabecera
        :param value: Valor de la cabecera
        """
        self.properties.headers[key] = value


class ContextFactory:
    """
    Factoria para la creacion de contextos
    """

    @classmethod
    def create_from_amqp_message(cls, message: AmqpMessage) -> Context:
        """
        Crea una instancia del contexto a partir del mensaje que ha llegado
        :param message:
        :return:
        """
        properties = AmqpMessageProperties.create()
        properties.headers = message.properties.headers
        properties.correlation_id = message.properties.correlation_id
        properties.app_id = message.properties.app_id

        return Context(properties)

    @classmethod
    def create_empty(cls) -> Context:
        """
        Crea una instancia del contexto vacia
        :return: Contexto
        :rtype: clitellum.services.Context
        """
        properties = AmqpMessageProperties.create()

        return Context(properties)


class PublisherConfiguration:

    def __init__(self, identification, config: ChannelConfiguration):
        self._sender_config = config
        self._identification = identification

    def get_sender_config(self) -> ChannelConfiguration:
        return self._sender_config

    def get_identification(self) -> Identification:
        return self._identification

class Publisher:
    """
    Publicador de mensajes
    """
    def __init__(self, config: PublisherConfiguration, channel: Channel):
        self._channel = channel
        self.config = config

    def connect(self):
        """
        Realiza la conexion del canal
        """
        logger_manager.get_logger().debug("Service - Conectando Canal de envio")
        self._channel.connect()

    def publish(self, message: str, routing_key: str, context: Context = None):
        """
        Publica un mensaje en el canal
        :param routing_key: Clave de enrutamiento
        :param message: Cuerpo del mensaje
        :param context: Contexto del mensaje
        """
        if context is None:
            context = ContextFactory.create_empty()

        logger_manager.get_logger().debug("Service - Publicando Mensaje")
        encoding = context.properties.content_encoding if context.properties.content_encoding is not None else 'utf-8'
        self._channel.publish(bytes(message, encoding), routing_key, context.properties)

    def close(self):
        """
        Cierra la conexion del canal
        """
        self._channel.close()


class PublisherFactory:
    """
    Factory de publicadores
    """
    @classmethod
    def create(cls, publisher_id: str, publisher_type: str, config: ChannelConfiguration) -> Publisher:
        """
        Crea una instancia de un publicador
        :param publisher_id: PublisherConfiguration
        :param publisher_type: PublisherConfiguration
        :param config: Configuracion del publicador
        :return:
        """
        channel = Channel.create(config)
        config = PublisherConfiguration(Identification(publisher_id, publisher_type), config)
        return Publisher(config, channel)


class ServiceConfiguration(PublisherConfiguration):

    def __init__(self, identification, sender_config: ChannelConfiguration, receiver_config: ChannelConfiguration):
        PublisherConfiguration.__init__(self, identification, sender_config)
        self._receiver_config = receiver_config

    def get_receiver_config(self) -> ChannelConfiguration:
        return self._receiver_config


class Service(Publisher):
    """
    Clase de Servicio
    """

    def __init__(self, config: ServiceConfiguration, channel_sender: Channel,
                 channel_receiver: Channel, handler_manager):
        Publisher.__init__(self, config, channel_sender)
        self.channel_receiver = channel_receiver
        self.channel_receiver.on_message_received = self._message_received
        self.handler_manager = handler_manager

    def run(self):
        self.connect()
        logger_manager.get_logger().debug("Service - Run %s - %s", self.config.get_identification().id,
                                          self.config.get_identification().type)
        self.channel_receiver.consume()

    def stop(self):
        logger_manager.get_logger().debug("Service - Stop - Waiting for channel")
        self.channel_receiver.stop_consuming()
        self.close()

    def connect(self):
        Publisher.connect(self)
        logger_manager.get_logger().debug("Service - Conectando Canal de recepcion")
        self.channel_receiver.connect()

    def close(self):
        Publisher.close(self)
        self.channel_receiver.close()

    def _send_error(self, message: bytes, ex: Exception):
        message_error = dict()
        message_error['description'] = str(ex)
        message_error['message'] = message
        str_error = json.dumps(message_error)
        self._channel.publish(bytes(str_error, 'utf-8'), 'Message.Error', AmqpMessageProperties.create())

    def _message_received(self, message: AmqpMessage):
        try:
            logger_manager.get_logger().debug("Service - Received Message")
            encoding = message.properties.content_encoding \
                if message.properties.content_encoding is not None else 'utf-8'

            message_str = str(message.body, encoding=encoding)
            handler_class = self.handler_manager.get_handler(message.frame.routing_key)
            handler = handler_class()
            handler.initialize(self,
                               ContextFactory.create_from_amqp_message(message),
                               message_str)
            handler.handle()

        except Exception as ex:
            logger_manager.get_logger().error("Service - Error al procesar el mensaje", ex)
            self._send_error(message.body, ex)
        finally:
            self.channel_receiver.ack(message)


class ServiceFactory:
    """
    Factoria para los servicios
    """

    @classmethod
    def create_service(cls, service_id: str, service_type: str, config_sender: ChannelConfiguration,
                       config_receiver: ChannelConfiguration, handler_manager) -> Service:
        """
        Crea una instancia de un servicio
        :param handler_manager: controlador de los handlers de los mensajes
        :param config_receiver: configuracion del receptor
        :param config_sender: configuracion del publicador
        :param service_id: identificador del servicio
        :param service_type: tipo del servicio
        :return: Service
        :rtype: clitellum.services.Service
        """
        channel_sender = Channel.create(config_sender)
        channel_receiver = Channel.create(config_receiver)
        service_config = ServiceConfiguration(Identification(service_id, service_type), config_sender, config_receiver)
        return Service(service_config, channel_sender, channel_receiver, handler_manager)
